# Generated by Django 3.0.14 on 2021-10-07 09:55

import core.fields
import core.models
import datetime
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0014_users'),
    ]

    operations = [
        migrations.CreateModel(
            name='Payer',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='PayerID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='PayerUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('type', models.CharField(choices=[('C', 'Co-operative'), ('D', 'Donor'), ('G', 'Government'), ('L', 'Local Authority'), ('O', 'Other'), ('P', 'Private Organization')], db_column='PayerType', max_length=1)),
                ('name', models.CharField(db_column='PayerName', max_length=100)),
                ('address', models.CharField(blank=True, db_column='PayerAddress', max_length=100, null=True)),
                ('phone', models.CharField(blank=True, db_column='Phone', max_length=50, null=True)),
                ('fax', models.CharField(blank=True, db_column='Fax', max_length=50, null=True)),
                ('email', models.CharField(blank=True, db_column='eMail', max_length=50, null=True)),
                ('audit_user_id', models.IntegerField(db_column='AuditUserID')),
            ],
            options={
                'db_table': 'tblPayer',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='PayerType',
            fields=[
                ('code', models.CharField(db_column='Code', max_length=1, primary_key=True, serialize=False)),
                ('payer_type', models.CharField(db_column='PayerType', max_length=50)),
                ('alt_language', models.CharField(blank=True, db_column='AltLanguage', max_length=50, null=True)),
                ('sort_order', models.IntegerField(blank=True, db_column='SortOrder', null=True)),
            ],
            options={
                'db_table': 'tblPayerType',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='PayerMutation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('mutation', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='payers', to='core.MutationLog')),
                ('payer', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='payer.Payer')),
            ],
            options={
                'db_table': 'payer_PayerMutation',
                'managed': True,
            },
            bases=(models.Model, core.models.ObjectMutation),
        ),
    ]



# Generated by Django 3.2.19 on 2023-12-12 09:33

import core.fields
import dirtyfields.dirtyfields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import simple_history.models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0008_auto_20230510_1347'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('payer', '0003_set_managed_to_true'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalFunding',
            fields=[
                ('id', models.UUIDField(db_column='UUID', db_index=True, default=None, editable=False)),
                ('is_deleted', models.BooleanField(db_column='isDeleted', default=False)),
                ('json_ext', models.JSONField(blank=True, db_column='Json_ext', null=True)),
                ('date_created', core.fields.DateTimeField(db_column='DateCreated', null=True)),
                ('date_updated', core.fields.DateTimeField(db_column='DateUpdated', null=True)),
                ('version', models.IntegerField(default=1)),
                ('amount', models.DecimalField(blank=True, db_column='Amount', decimal_places=2, max_digits=18, null=True)),
                ('pay_date', models.DateField(blank=True, db_column='PaidDate', null=True)),
                ('status', models.CharField(choices=[('N', 'PENDING'), ('P', 'PAID'), ('A', 'AWAITING_FOR_RECONCILIATION'), ('R', 'RECONCILIATED')], db_column='Status', default='N', max_length=1)),
                ('receipt', models.CharField(db_column='Receipt', max_length=50)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField(db_index=True)),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('payer', models.ForeignKey(blank=True, db_column='PayerID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='payer.payer')),
                ('product', models.ForeignKey(blank=True, db_column='ProdID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='product.product')),
                ('user_created', models.ForeignKey(blank=True, db_column='UserCreatedUUID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('user_updated', models.ForeignKey(blank=True, db_column='UserUpdatedUUID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical funding',
                'verbose_name_plural': 'historical fundings',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': ('history_date', 'history_id'),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='Funding',
            fields=[
                ('id', models.UUIDField(db_column='UUID', default=None, editable=False, primary_key=True, serialize=False)),
                ('is_deleted', models.BooleanField(db_column='isDeleted', default=False)),
                ('json_ext', models.JSONField(blank=True, db_column='Json_ext', null=True)),
                ('date_created', core.fields.DateTimeField(db_column='DateCreated', null=True)),
                ('date_updated', core.fields.DateTimeField(db_column='DateUpdated', null=True)),
                ('version', models.IntegerField(default=1)),
                ('amount', models.DecimalField(blank=True, db_column='Amount', decimal_places=2, max_digits=18, null=True)),
                ('pay_date', models.DateField(blank=True, db_column='PaidDate', null=True)),
                ('status', models.CharField(choices=[('N', 'PENDING'), ('P', 'PAID'), ('A', 'AWAITING_FOR_RECONCILIATION'), ('R', 'RECONCILIATED')], db_column='Status', default='N', max_length=1)),
                ('receipt', models.CharField(db_column='Receipt', max_length=50)),
                ('payer', models.ForeignKey(blank=True, db_column='PayerID', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='fundings', to='payer.payer')),
                ('product', models.ForeignKey(blank=True, db_column='ProdID', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='fundings', to='product.product')),
                ('user_created', models.ForeignKey(db_column='UserCreatedUUID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='funding_user_created', to=settings.AUTH_USER_MODEL)),
                ('user_updated', models.ForeignKey(db_column='UserUpdatedUUID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='funding_user_updated', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'tblFunding',
                'managed': True,
            },
            bases=(dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
    ]



import graphene
from graphene_django import DjangoObjectType
from django.db.models import Q
from django import forms
from django.utils.translation import gettext_lazy

from invoice.apps import InvoiceConfig
from .apps import PayerConfig
from .models import Payer, Funding
from core import ExtendedConnection
from core.models import filter_validity

from product.schema import ProductGQLType
import django_filters
from django.utils.translation import gettext as _
from django.core.exceptions import PermissionDenied


class IntegerFilter(django_filters.NumberFilter):
    field_class = forms.IntegerField
class CharFilter(django_filters.CharFilter):
    field_class = forms.CharField


class UUIDFilter(django_filters.UUIDFilter):
    field_class = forms.UUIDField
    
class FundingFilter(django_filters.FilterSet):
    uuid = UUIDFilter(
        lookup_expr=["exact"],
        method="filter_uuid",
        label=gettext_lazy("Filter funding by UUID"),
    )
    
    def filter_uuid(self, queryset, name, value):
        return queryset.filter(id = value)
    
    class Meta:
        model = Funding
        fields = {
            "id": ["exact"],
            "product": ["exact"],
            "pay_date": ["exact", "lt", "lte", "gt", "gte"],
            "receipt": ["exact", "icontains"],
            "amount": ["exact", "lt", "lte", "gt", "gte"],
            "status": ["exact"],

        }
class PayerFilter(django_filters.FilterSet):
    location = IntegerFilter(
        lookup_expr=["exact"],
        method="filter_location",
        label=gettext_lazy("Filter payers with or below a given location ID"),
    )
    type = CharFilter(
        lookup_expr=["exact"],
        method="filter_type",
        label=gettext_lazy("Filter payers with or below a given payer type code"),
    )

    def filter_location(self, queryset, name, value):
        return queryset.filter(Q(location__id=value) | Q(location__parent__id=value))
    def filter_type(self, queryset, name, value):
            return queryset.filter(type=value)
    class Meta:
        model = Payer
        fields = {
            "id": ["exact"],
            "uuid": ["exact"],
            "name": ["exact", "icontains"],
            "email": ["exact", "icontains"],
            "phone": ["exact", "icontains"],
        }


class FundingGQLType(DjangoObjectType):
    uuid = graphene.UUID()
    
    def resolve_uuid(self, info, **kwargs):
        return self.id
    class Meta:
        model = Funding
        interfaces = (graphene.relay.Node,)
        filterset_class = FundingFilter
        connection_class = ExtendedConnection

class FundingConnection(graphene.Connection):
    class Meta:
        node = FundingGQLType

    total_count = graphene.Int()

    def resolve_total_count(self, info, **kwargs):
        if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
            raise PermissionDenied(_("unauthorized"))
        return len(self.iterable)


class PayerGQLType(DjangoObjectType):

    #premiums = graphene.relay.ConnectionField(FundingConnection)
    fundings = graphene.relay.ConnectionField(FundingConnection)

    def resolve_fundings(self, info, **kwargs):
        if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
            raise PermissionDenied(_("unauthorized"))
        return (
            self.fundings.order_by("-pay_date").all()
        )

    def resolve_premiums(self, info, **kwargs):
        if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
            raise PermissionDenied(_("unauthorized"))
        return (
            self.premuims.fitler(*filter_validity()).order_by("-pay_date").all()
        )

    class Meta:
        model = Payer
        interfaces = (graphene.relay.Node,)
        filterset_class = PayerFilter
        connection_class = ExtendedConnection

        exclude_fields = ("premiums",)



from gettext import gettext as _
import logging
import graphene
from graphene.relay import Node
from core.schema import OpenIMISMutation
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import ValidationError, PermissionDenied
from .apps import PayerConfig
from location.models import Location
from .models import Payer, PayerMutation, Funding


logger = logging.getLogger(__name__)


def create_or_update_payer(user, data):
    client_mutation_id = data.pop("client_mutation_id", None)
    data.pop("client_mutation_label", None)
    payer_uuid = data.pop("uuid", None)
    location_uuid = data.pop("location_uuid", None)

    if payer_uuid:
        payer = Payer.objects.get(uuid=payer_uuid)
        if payer.validity_to:
            raise ValidationError(_('Cannot update historical values'))
        payer.save_history()
        for (key, value) in data.items():
            setattr(payer, key, value)
    else:
        payer = Payer.objects.create(**data)

    if location_uuid is not None:
        payer.location = Location.objects.get(uuid=location_uuid)

    payer.save()

    if client_mutation_id:
        PayerMutation.object_mutated(
            user, client_mutation_id=client_mutation_id, payer=payer
        )


class CreateOrUpdatePayerMutation(OpenIMISMutation):
    @classmethod
    def do_mutate(cls, perms, user, **data):
        if type(user) is AnonymousUser or not user.id:
            raise ValidationError(_("mutation.authentication_required"))
        if not user.has_perms(perms):
            raise PermissionDenied(_("unauthorized"))

        data["audit_user_id"] = user.id_for_audit

        return create_or_update_payer(user, data)


class PayerInputType(OpenIMISMutation.Input):
    name = graphene.String(required=True)
    email = graphene.String()
    phone = graphene.String()
    fax = graphene.String()
    address = graphene.String()
    location_uuid = graphene.UUID(required=True)
    type = graphene.String(required=True)


class CreatePayerMutation(CreateOrUpdatePayerMutation):
    _mutation_module = "payer"
    _mutation_class = "CreatePayerMutation"

    class Input(PayerInputType):
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        try:
            cls.do_mutate(
                PayerConfig.gql_mutation_payer_add_perms,
                user,
                **data,
            )
        except Exception as exc:
            logger.exception(exc)
            return [
                {
                    "message": _("payer.mutation.failed_to_create_payer"),
                    "detail": str(exc),
                }
            ]


class UpdatePayerMutation(CreateOrUpdatePayerMutation):
    _mutation_module = "payer"
    _mutation_class = "UpdatePayerMutation"

    class Input(PayerInputType):
        uuid = graphene.UUID(required=True)

    @classmethod
    def async_mutate(cls, user, **data):
        try:
            cls.do_mutate(
                PayerConfig.gql_mutation_payer_update_perms,
                user,
                **data,
            )
        except Exception as exc:
            logger.exception(exc)
            return [
                {
                    "message": _("payer.mutation.failed_to_update_payer")
                    % {"uuid": data["uuid"]},
                    "detail": str(exc),
                }
            ]


class DeletePayerMutation(OpenIMISMutation):
    _mutation_module = "payer"
    _mutation_class = "DeletePayerMutation"

    class Input(OpenIMISMutation.Input):
        uuids = graphene.List(graphene.String)

    @classmethod
    def async_mutate(cls, user, **data):
        if not user.has_perms(PayerConfig.gql_mutation_payer_delete_perms):
            raise PermissionDenied(_("unauthorized"))
        errors = []

        for uuid in data["uuids"]:
            obj = Payer.objects.filter(uuid=uuid).first()
            if obj is None:
                errors.append(
                    {
                        "title": uuid,
                        "list": [
                            {
                                "message": _("payer.validation.id_does_not_exist")
                                % {"id", uuid}
                            }
                        ],
                    }
                )
                continue
            try:
                obj.delete_history()
            except Exception as exc:
                logger.exception(exc)
                errors.append(
                    {
                        "title": uuid,
                        "list": [
                            {
                                "message": _("payer.mutation.failed_to_delete_payer")
                                % {"uuid": obj.uuid},
                                "detail": str(exc),
                            }
                        ],
                    }
                )

        if len(errors) == 1:
            errors = errors[0]["list"]
        return errors


class AddFundingMutation(OpenIMISMutation):
    _mutation_module = "payer"
    _mutation_class = "AddFundingMutation"

    class Input(OpenIMISMutation.Input):
        payer_id = graphene.ID(required=True)
        product_id = graphene.ID(required=True)
        pay_date = graphene.Date(required=True)
        amount = graphene.Decimal(required=True)
        receipt = graphene.String(required=True)

    @classmethod
    def async_mutate(cls, user, **data):
        client_mutation_id = data.get("client_mutation_id", None)
        if not user.has_perms(PayerConfig.gql_mutation_payer_update_perms):
            raise PermissionDenied(_("unauthorized"))

        from product.models import Product

        payer = Payer.objects.filter(
            validity_to__isnull=True, id=data.get("payer_id")
        ).get()
        product = Product.objects.filter(
            validity_to__isnull=True, id=data.get("product_id")
        ).get()

        try:
            funding = Funding(**{
                'payer':payer,
                'product': product,
                'pay_date':data.get("pay_date"),
                'amount':data.get("amount"),
                'receipt':data.get("receipt"),
            }
            )
            funding.save(username = user.username)

        except Exception as exc:
            logger.exception(exc)
            return [
                {
                    "message": _("payer.mutation.failed_to_add_funding"),
                    "detail": str(exc),
                }
            ]



import graphene
from django.db.models import Q
from django.core.exceptions import PermissionDenied
import graphene_django_optimizer as gql_optimizer
from core import filter_validity
from django.utils.translation import gettext as _, gettext_lazy
from payer.apps import PayerConfig
from core.schema import OrderedDjangoFilterConnectionField
from .models import Payer
from location.models import Location, LocationManager
from product.schema import ProductGQLType

from .gql_queries import PayerGQLType, FundingGQLType

from .gql_mutations import (
    CreatePayerMutation,
    UpdatePayerMutation,
    DeletePayerMutation,
    AddFundingMutation,
)


class Query(graphene.ObjectType):
    fundings = OrderedDjangoFilterConnectionField(
        FundingGQLType,
        payer_id=graphene.ID(),
        client_mutation_id=graphene.String(),
        show_history=graphene.Boolean(),
        parent_location_level=graphene.Int(),
        orderBy=graphene.List(of_type=graphene.String),
    )
    
    funding = graphene.Field(FundingGQLType, uuid=graphene.UUID())

        
    payers = OrderedDjangoFilterConnectionField(
        PayerGQLType,
        show_history=graphene.Boolean(),
        search=graphene.String(
            description=gettext_lazy("Search in `name`, `phone` & `email`")
        ),
        orderBy=graphene.List(of_type=graphene.String),
    )
    payer = graphene.Field(PayerGQLType, uuid=graphene.UUID())

    def resolve_payer(self, info, uuid, **kwargs):
        if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
            raise PermissionDenied(_("unauthorized"))

        return Payer.objects.get(uuid=uuid)

    def resolve_payers(self, info, **kwargs):
        if not info.context.user.has_perms(PayerConfig.gql_query_payers_perms):
            raise PermissionDenied(_("unauthorized"))

        filters = Payer.objects
        show_history = kwargs.get("show_history", False)
        if not show_history:
            filters = filters.filter(*filter_validity(**kwargs))

        search = kwargs.get("search", None)
        if search is not None:
            filters = filters.filter(
                Q(name__icontains=search)
                | Q(phone__icontains=search)
                | Q(email__icontains=search)
            )

        filters = LocationManager().build_user_location_filter_query(info.context.user._u, queryset = filters)

        return gql_optimizer.query(filters, info)


class Mutation(graphene.ObjectType):
    create_payer = CreatePayerMutation.Field()
    update_payer = UpdatePayerMutation.Field()
    delete_payer = DeletePayerMutation.Field()
    add_funding = AddFundingMutation.Field()



from django.apps import AppConfig

MODULE_NAME = "payer"

DEFAULT_CFG = {
    "gql_query_payers_perms": ["121801"],
    "gql_mutation_payer_add_perms": ["121802"],
    "gql_mutation_payer_update_perms": ["122103"],
    "gql_mutation_payer_delete_perms": ["121804"],
}


class PayerConfig(AppConfig):
    name = MODULE_NAME

    gql_query_payers_perms = []
    gql_mutation_payer_add_perms = []
    gql_mutation_payer_update_perms = []
    gql_mutation_payer_delete_perms = []

    def __load_config(self, cfg):
        for field in cfg:
            if hasattr(PayerConfig, field):
                setattr(PayerConfig, field, cfg[field])

    def ready(self):
        from core.models import ModuleConfiguration

        cfg = ModuleConfiguration.get_or_default(MODULE_NAME, DEFAULT_CFG)
        self.__load_config(cfg)



import uuid
import itertools
from django.conf import settings
from core import models as core_models
from django.db import models
from django.utils.translation import gettext as _
from product.models import Product

class PayerType(models.Model):
    code = models.CharField(db_column="Code", primary_key=True, max_length=1)
    payer_type = models.CharField(db_column="PayerType", max_length=50)
    alt_language = models.CharField(
        db_column="AltLanguage", max_length=50, blank=True, null=True
    )
    sort_order = models.IntegerField(db_column="SortOrder", blank=True, null=True)

    class Meta:
        managed = True
        db_table = "tblPayerType"


class Payer(core_models.VersionedModel):
    PAYER_TYPE_COOP = "C"
    PAYER_TYPE_DONOR = "D"
    PAYER_TYPE_GOV = "G"
    PAYER_TYPE_LOCAL_AUTH = "L"
    PAYER_TYPE_OTHER = "O"
    PAYER_TYPE_PRIVATE_ORG = "P"
    PAYER_TYPE_CHOICES = (
        (PAYER_TYPE_COOP, "Co-operative"),
        (PAYER_TYPE_DONOR, "Donor"),
        (PAYER_TYPE_GOV, "Government"),
        (PAYER_TYPE_LOCAL_AUTH, "Local Authority"),
        (PAYER_TYPE_OTHER, "Other"),
        (PAYER_TYPE_PRIVATE_ORG, "Private Organization"),
    )

    id = models.AutoField(db_column="PayerID", primary_key=True)
    uuid = models.CharField(
        db_column="PayerUUID", max_length=36, default=uuid.uuid4, unique=True
    )
    type = models.CharField(
        db_column="PayerType", max_length=1, choices=PAYER_TYPE_CHOICES
    )
    name = models.CharField(db_column="PayerName", max_length=100, null=False)
    address = models.CharField(
        db_column="PayerAddress", max_length=100, null=True, blank=True
    )
    location = models.ForeignKey(
        "location.Location",
        db_column="LocationId",
        blank=True,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="+",
    )
    phone = models.CharField(db_column="Phone", max_length=50, null=True, blank=True)
    fax = models.CharField(db_column="Fax", max_length=50, null=True, blank=True)
    email = models.CharField(db_column="eMail", max_length=50, null=True, blank=True)

    audit_user_id = models.IntegerField(db_column="AuditUserID")

    # rowid = models.TextField(db_column='RowID', blank=True, null=True)

    @classmethod
    def get_queryset(cls, queryset, user):
        from location.models import LocationManager

        queryset = cls.filter_queryset(queryset)
        if settings.ROW_SECURITY and not user._u.is_imis_admin:
            queryset = LocationManager().build_user_location_filter_query(user._u, queryset = Payer.objects, loc_types=['R', 'D'])
        return queryset

    class Meta:
        managed = True
        db_table = "tblPayer"

class Funding(core_models.HistoryModel):
    class FundingStatus(models.TextChoices):
        PENDING = "N", _("PENDING")
        PAID = "P", _("PAID")
        AWAITING_FOR_RECONCILIATION = "A", _("AWAITING_FOR_RECONCILIATION")
        RECONCILIATED = "R", _("RECONCILIATED")

    product = models.ForeignKey(Product,
                                models.DO_NOTHING, db_column='ProdID',
                                blank=True, null=True,
                                related_name="fundings")
    amount = models.DecimalField(
        db_column='Amount', max_digits=18, decimal_places=2, blank=True, null=True)
    pay_date = models.DateField(
        db_column='PaidDate', blank=True, null=True)
    status = models.CharField(
        db_column='Status', max_length=1, choices=FundingStatus.choices, default=FundingStatus.PENDING, null=False
    )
    payer = models.ForeignKey(
        Payer,
        models.DO_NOTHING,
        db_column="PayerID",
        blank=True,
        null=True,
        related_name="fundings",
    )
    receipt = models.CharField(db_column="Receipt", max_length=50)
    
    class Meta:
        managed = True
        db_table = "tblFunding"
        
class PayerMutation(core_models.UUIDModel, core_models.ObjectMutation):
    payer = models.ForeignKey(Payer, models.DO_NOTHING, related_name="+")
    mutation = models.ForeignKey(
        core_models.MutationLog, models.DO_NOTHING, related_name="payers"
    )

    class Meta:
        managed = True
        db_table = "payer_PayerMutation"



# Generated by Django 3.2.18 on 2023-05-12 14:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('payer', '0001_initial'),
        ('location', '0013_auto_20230317_1534'),
    ]

    operations = [
        migrations.AddField(
            model_name='payer',
            name='location',
            field=models.ForeignKey(blank=True, db_column='LocationId', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='location.location'),
        ),
    ]



# Generated by Django 3.2.18 on 2023-05-12 14:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('payer', '0002_add_missing_fields'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='payer',
            options={'managed': True},
        ),
        migrations.AlterModelOptions(
            name='payertype',
            options={'managed': True},
        ),
    ]



import base64
import json
from dataclasses import dataclass

from core.models import User
from core.test_helpers import create_test_interactive_user
from django.conf import settings
from graphene_django.utils.testing import GraphQLTestCase
from graphql_jwt.shortcuts import get_token
from location.models import Location
from location.test_helpers import create_test_location, assign_user_districts
from rest_framework import status
from insuree.test_helpers import create_test_insuree
from location.test_helpers import create_test_location, create_test_health_facility, create_test_village
from payer.models import Payer
from product.models import Product


# from openIMIS import schema

from core.models.openimis_graphql_test_case import openIMISGraphQLTestCase, BaseTestContext as DummyContext

class PayerGQLTestCase(openIMISGraphQLTestCase):
    GRAPHQL_URL = f'/{settings.SITE_ROOT()}graphql'
    # This is required by some version of graphene but is never used. It should be set to the schema but the import
    # is shown as an error in the IDE, so leaving it as True.
    GRAPHQL_SCHEMA = True
    admin_user = None
    ca_user = None
    ca_token = None
    test_village = None
    test_insuree = None
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.test_village = create_test_village()
        cls.test_insuree = create_test_insuree(with_family=True, is_head=True, custom_props={'current_village':cls.test_village}, family_custom_props={'location':cls.test_village})
        cls.admin_user = create_test_interactive_user(username="testLocationAdmin")
        cls.admin_token = get_token(cls.admin_user, DummyContext(user=cls.admin_user))
        cls.ca_user = create_test_interactive_user(username="testLocationNoRight", roles=[9])
        cls.ca_token = get_token(cls.ca_user, DummyContext(user=cls.ca_user))
        cls.admin_dist_user = create_test_interactive_user(username="testLocationDist")
        assign_user_districts(cls.admin_dist_user, ["R1D1", "R2D1", "R2D2", "R2D1", cls.test_village.parent.parent.code])
        cls.admin_dist_token = get_token(cls.admin_dist_user, DummyContext(user=cls.admin_dist_user))

    def test_query_payer(self):
        response = self.query(
            '''
        query usePayersQuery (
          $first: Int, $last: Int, $before: String, $after: String, $phone: String, $name: String,
          $email: String, $location: Int, $showHistory: Boolean, $search: String, $type: String,
          ) {
          payers (
            first: $first, last: $last, before: $before, after: $after, phone_Icontains: $phone, showHistory: $showHistory, type: $type
            name_Icontains: $name, location: $location, email_Icontains: $email, search: $search
            ) {
            edges {
              node {
                id
                uuid
                ...PayerFragment
              }
            }
            pageInfo {
              hasNextPage
              hasPreviousPage
              startCursor
              endCursor
            }
            totalCount
          }
        }

        fragment PayerFragment on PayerGQLType {
          id
          uuid
          name
          email
          phone
          type
          address
          location {id name uuid code parent {id name uuid code}}
          validityFrom
          validityTo
        } 
            ''',
            headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"},
            variables={ 'first':10, 'type':'C'},
        )

        self.assertEquals(response.status_code, status.HTTP_200_OK)
        content = json.loads(response.content)

        self.assertResponseNoErrors(response)
        
        
    def test_add_funding(self):
      
        response = self.query(
      '''
    mutation useAddFundingMutation($input: AddFundingMutationInput!) {
      addFunding(input: $input) {
        internalId
        clientMutationId
      }
    }
  
      ''',
            headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"},
            variables={
              "input": {
                "amount": 34576,
                "clientMutationId": "6e3747b2-135b-4258-ab2b-d00bb2c4f640",
                "payDate": "2023-12-12",
                "payerId": Payer.objects.first().id,
                "productId": Product.objects.first().id,
                "receipt": "324534"
              }
            },
        )

        self.assertEquals(response.status_code, status.HTTP_200_OK)
        content = json.loads(response.content)

        self.assertResponseNoErrors(response)
        #wait 
        
        response = self.query('''
        
        {
        mutationLogs(clientMutationId: "6e3747b2-135b-4258-ab2b-d00bb2c4f640")
        {
            
        pageInfo { hasNextPage, hasPreviousPage, startCursor, endCursor}
        edges
        {
        node
        {
            id,status,error,clientMutationId,clientMutationLabel,clientMutationDetails,requestDateTime,jsonExt
        }
        }
        }
        }
        
        ''',
            headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"})
        
        self.assertEquals(response.status_code, status.HTTP_200_OK)
        content = json.loads(response.content)
        
    def test_query_funding(self):
        response = self.query(
        '''
        query usePayerFundingsQuery (
          $first: Int, $last: Int, $before: String, $after: String, $payerId: UUID!
          ) {
          payer (uuid: $payerId) {
            fundings (first: $first, last: $last, before: $before, after: $after) {
              edges {
                node {
                  uuid
                  amount
                  payDate
                  product {
                    name
                  }
                  receipt
                }
              }
              pageInfo {
                hasNextPage
                hasPreviousPage
                startCursor
                endCursor
              }
              totalCount
            }
          }
        }
        ''',
            headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"},
            variables={ 'first':10, 'payerId':Payer.objects.first().uuid},
        )

        self.assertEquals(response.status_code, status.HTTP_200_OK)
        content = json.loads(response.content)

        self.assertResponseNoErrors(response)

        







# Generated by Django 3.2.16 on 2023-05-04 11:56
import logging

from django.db import migrations
from core.utils import insert_role_right_for_system
from payer.apps import DEFAULT_CFG

DEFAULT_CFG = {
    "gql_query_payers_perms": ["121801"],
    "gql_mutation_payer_add_perms": ["121802"],
    "gql_mutation_payer_update_perms": ["122103"],
    "gql_mutation_payer_delete_perms": ["121804"],
}



logger = logging.getLogger(__name__)

EO_ROLE_IS_SYSTEM = 1
SCHEME_ADMIN_ROLE_IS_SYSTEM = 32
PAYER_SEARCH_ROLE_RIGHT_ID = DEFAULT_CFG["gql_query_payers_perms"][0]
PAYER_UPDATE_ROLE_RIGHT_ID = DEFAULT_CFG["gql_mutation_payer_update_perms"][0]
PAYER_DELETE_ROLE_RIGHT_ID = DEFAULT_CFG["gql_mutation_payer_delete_perms"][0]
PAYER_CREATE_ROLE_RIGHT_ID = DEFAULT_CFG["gql_mutation_payer_add_perms"][0]



def add_rights(apps, schema_editor):
    """
    Add subscription CRUD permission to the EO and scheme admin.
    """
    insert_role_right_for_system(EO_ROLE_IS_SYSTEM, PAYER_SEARCH_ROLE_RIGHT_ID, apps)
    insert_role_right_for_system(SCHEME_ADMIN_ROLE_IS_SYSTEM, PAYER_SEARCH_ROLE_RIGHT_ID, apps)
    insert_role_right_for_system(SCHEME_ADMIN_ROLE_IS_SYSTEM, PAYER_UPDATE_ROLE_RIGHT_ID, apps)
    insert_role_right_for_system(SCHEME_ADMIN_ROLE_IS_SYSTEM, PAYER_DELETE_ROLE_RIGHT_ID, apps)
    insert_role_right_for_system(SCHEME_ADMIN_ROLE_IS_SYSTEM, PAYER_CREATE_ROLE_RIGHT_ID, apps)
    

def remove_rights(apps, schema_editor):
    RoleRight = apps.get_model('core', 'RoleRight')
    Role = apps.get_model('core', 'Role')
    """
    Remove subscription CRUD permissions to the EO and scheme admin.
    """
    RoleRight.objects.filter(
        role__is_system__in=[EO_ROLE_IS_SYSTEM, SCHEME_ADMIN_ROLE_IS_SYSTEM],
        right_id__in=[PAYER_SEARCH_ROLE_RIGHT_ID, PAYER_CREATE_ROLE_RIGHT_ID, PAYER_DELETE_ROLE_RIGHT_ID, PAYER_UPDATE_ROLE_RIGHT_ID],
        validity_to__isnull=True
    ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('payer', '0005_alter_funding_date_created_and_more'),
    ]

    operations = [
        migrations.RunPython(add_rights, remove_rights),
    ]
